@@ -27,6 +27,7 @@ import java.io.*;
 import javax.swing.*;
 
 import net.sourceforge.jnlp.*;
+import net.sourceforge.jnlp.splashscreen.SplashController;
 import net.sourceforge.jnlp.util.*;
 
 /**
@@ -130,6 +131,15 @@ public class AppletEnvironment implements AppletContext, AppletStub {
         return cont;
     }
 
+     /**
+     * container must be SplashContoler
+     * 
+     */
+    public SplashController getSplashControler() {
+        
+        return (SplashController)cont;
+    }
+
     /**
      * Initialize, start, and show the applet.
      */
