@@ -63,6 +63,7 @@ import java.util.jar.JarEntry;
 import net.sourceforge.jnlp.util.JarFile;
 import java.util.jar.Manifest;
 
+import net.sourceforge.jnlp.security.appletextendedsecurity.UnsignedAppletTrustConfirmation;
 import net.sourceforge.jnlp.AppletDesc;
 import net.sourceforge.jnlp.ApplicationDesc;
 import net.sourceforge.jnlp.DownloadOptions;
@@ -73,6 +74,7 @@ import net.sourceforge.jnlp.JNLPMatcher;
 import net.sourceforge.jnlp.JNLPMatcherException;
 import net.sourceforge.jnlp.LaunchDesc;
 import net.sourceforge.jnlp.LaunchException;
+import net.sourceforge.jnlp.NullJnlpFileException;
 import net.sourceforge.jnlp.ParseException;
 import net.sourceforge.jnlp.PluginBridge;
 import net.sourceforge.jnlp.ResourcesDesc;
@@ -82,10 +84,13 @@ import net.sourceforge.jnlp.cache.CacheUtil;
 import net.sourceforge.jnlp.cache.IllegalResourceDescriptorException;
 import net.sourceforge.jnlp.cache.ResourceTracker;
 import net.sourceforge.jnlp.cache.UpdatePolicy;
+import net.sourceforge.jnlp.security.AppVerifier;
+import net.sourceforge.jnlp.security.JNLPAppVerifier;
+import net.sourceforge.jnlp.security.PluginAppVerifier;
 import net.sourceforge.jnlp.security.SecurityDialogs;
-import net.sourceforge.jnlp.security.SecurityDialogs.AccessType;
 import net.sourceforge.jnlp.tools.JarCertVerifier;
 import net.sourceforge.jnlp.util.FileUtils;
+import net.sourceforge.jnlp.util.StreamUtils;
 import sun.misc.JarIndex;
 
 /**
@@ -107,6 +112,11 @@ public class JNLPClassLoader extends URLClassLoader {
     final public static String TEMPLATE = "JNLP-INF/APPLICATION_TEMPLATE.JNLP";
     final public static String APPLICATION = "JNLP-INF/APPLICATION.JNLP";
     
+    /** Actions to specify how cache is to be managed **/
+    public static enum DownloadAction {
+        DOWNLOAD_TO_CACHE, REMOVE_FROM_CACHE, CHECK_CACHE
+    }
+
     /** True if the application has a signed JNLP File */
     private boolean isSignedJNLP = false;
     
@@ -159,14 +169,8 @@ public class JNLPClassLoader extends URLClassLoader {
     /** all jars not yet part of classloader or active */
     private List<JARDesc> available = new ArrayList<JARDesc>();
 
-    /** all of the jar files that were verified */
-    private ArrayList<String> verifiedJars = null;
-
-    /** all of the jar files that were not verified */
-    private ArrayList<String> unverifiedJars = null;
-
     /** the jar cert verifier tool to verify our jars */
-    private JarCertVerifier jcv = null;
+    private final JarCertVerifier jcv;
 
     private boolean signing = false;
 
@@ -214,7 +218,8 @@ public class JNLPClassLoader extends URLClassLoader {
      * Create a new JNLPClassLoader from the specified file.
      *
      * @param file the JNLP file
-     * @param name of the application's main class
+     * @param policy the UpdatePolicy for this class loader
+     * @param mainName name of the application's main class
      */
     protected JNLPClassLoader(JNLPFile file, UpdatePolicy policy, String mainName) throws LaunchException {
         super(new URL[0], JNLPClassLoader.class.getClassLoader());
@@ -228,6 +233,16 @@ public class JNLPClassLoader extends URLClassLoader {
 
         this.mainClass = mainName;
 
+        AppVerifier verifier;
+
+        if (file instanceof PluginBridge && !((PluginBridge)file).useJNLPHref()) {
+            verifier = new PluginAppVerifier();
+        } else {
+            verifier = new JNLPAppVerifier();
+        }
+
+        jcv = new JarCertVerifier(verifier);
+
         // initialize extensions
         initializeExtensions();
 
@@ -316,7 +331,11 @@ public class JNLPClassLoader extends URLClassLoader {
              *
              */
             if (!file.getSecurity().getSecurityType().equals(SecurityDesc.SANDBOX_PERMISSIONS) && !signing) {
-                throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LUnsignedJarWithSecurity"), R("LUnsignedJarWithSecurityInfo"));
+                if (jcv.allJarsSigned()) {
+                    throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LSignedJNLPAppDifferentCerts"), R("LSignedJNLPAppDifferentCertsInfo"));
+                } else {
+                    throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LUnsignedJarWithSecurity"), R("LUnsignedJarWithSecurityInfo"));
+                }
             } else if (signing == true) {
                 this.security = file.getSecurity();
             } else {
@@ -327,16 +346,6 @@ public class JNLPClassLoader extends URLClassLoader {
         }
     }
 
-    /**
-     * Returns a JNLP classloader for the specified JNLP file.
-     *
-     * @param file the file to load classes for
-     * @param policy the update policy to use when downloading resources
-     */
-    public static JNLPClassLoader getInstance(JNLPFile file, UpdatePolicy policy) throws LaunchException {
-        return getInstance(file, policy, null);
-    }
-
     /**
      * Gets the lock for a given unique key, creating one if it does not yet exist.
      * This operation is atomic & thread-safe.
@@ -357,6 +366,60 @@ public class JNLPClassLoader extends URLClassLoader {
         }
     }
 
+    /**
+     * Creates a fully initialized JNLP classloader for the specified JNLPFile, 
+     * to be used as an applet/application's classloader.
+     * In contrast, JNLP classloaders can also be constructed simply to merge 
+     * its resources into another classloader.
+     *
+     * @param file the file to load classes for
+     * @param policy the update policy to use when downloading resources
+     * @param mainName Overrides the main class name of the application
+     */
+    private static JNLPClassLoader createInstance(JNLPFile file, UpdatePolicy policy, String mainName) throws LaunchException {
+        String uniqueKey = file.getUniqueKey();
+        JNLPClassLoader baseLoader = uniqueKeyToLoader.get(uniqueKey);
+        JNLPClassLoader loader = new JNLPClassLoader(file, policy, mainName);
+
+        // If security level is 'high' or greater, we must check if the user allows unsigned applets 
+        // when the JNLPClassLoader is created. We do so here, because doing so in the constructor 
+        // causes unwanted side-effects for some applets
+        if (!loader.getSigning() && file instanceof PluginBridge) {
+            UnsignedAppletTrustConfirmation.checkUnsignedWithUserIfRequired((PluginBridge)file);
+        }
+
+        // New loader init may have caused extentions to create a
+        // loader for this unique key. Check.
+        JNLPClassLoader extLoader = uniqueKeyToLoader.get(uniqueKey);
+
+        if (extLoader != null && extLoader != loader) {
+            if (loader.signing && !extLoader.signing)
+                if (!SecurityDialogs.showNotAllSignedWarningDialog(file))
+                    throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LSignedAppJarUsingUnsignedJar"), R("LSignedAppJarUsingUnsignedJarInfo"));
+
+            loader.merge(extLoader);
+            extLoader.decrementLoaderUseCount(); // loader urls have been merged, ext loader is no longer used
+        }
+
+        // loader is now current + ext. But we also need to think of
+        // the baseLoader
+        if (baseLoader != null && baseLoader != loader) {
+           loader.merge(baseLoader);
+        }
+
+        return loader;
+    }
+
+    /**
+     * Returns a JNLP classloader for the specified JNLP file.
+     *
+     * @param file the file to load classes for
+     * @param policy the update policy to use when downloading resources
+     */
+    public static JNLPClassLoader getInstance(JNLPFile file, UpdatePolicy policy) throws LaunchException {
+        return getInstance(file, policy, null);
+    }
+
     /**
      * Returns a JNLP classloader for the specified JNLP file.
      *
@@ -378,27 +441,7 @@ public class JNLPClassLoader extends URLClassLoader {
                     (file.isApplication() && 
                      !baseLoader.getJNLPFile().getFileLocation().equals(file.getFileLocation()))) {
 
-                loader = new JNLPClassLoader(file, policy, mainName);
-
-                // New loader init may have caused extentions to create a
-                // loader for this unique key. Check.
-                JNLPClassLoader extLoader = uniqueKeyToLoader.get(uniqueKey);
-
-                if (extLoader != null && extLoader != loader) {
-                    if (loader.signing && !extLoader.signing)
-                        if (!SecurityDialogs.showNotAllSignedWarningDialog(file))
-                            throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LSignedAppJarUsingUnsignedJar"), R("LSignedAppJarUsingUnsignedJarInfo"));
-
-                    loader.merge(extLoader);
-                    extLoader.decrementLoaderUseCount(); // loader urls have been merged, ext loader is no longer used
-                }
-
-                // loader is now current + ext. But we also need to think of
-                // the baseLoader
-                if (baseLoader != null && baseLoader != loader) {
-                   loader.merge(baseLoader);
-                }
-
+                loader = createInstance(file, policy, mainName);
             } else {
                 // if key is same and locations match, this is the loader we want
                 if (!file.isApplication()) {
@@ -513,7 +556,7 @@ public class JNLPClassLoader extends URLClassLoader {
      * @param jar the jar to check
      * @return true if file exists AND is an invalid jar, false otherwise
      */
-    private boolean isInvalidJar(JARDesc jar){
+    boolean isInvalidJar(JARDesc jar){
         File cacheFile = tracker.getCacheFile(jar.getLocation());
         if (cacheFile == null)
             return false;//File cannot be retrieved, do not claim it is an invalid jar
@@ -561,9 +604,9 @@ public class JNLPClassLoader extends URLClassLoader {
 
         JARDesc jars[] = resources.getJARs();
 
-        if (jars == null || jars.length == 0) {
+        if (jars.length == 0) {
 
-            boolean allSigned = true;
+            boolean allSigned = (loaders.length > 1) /* has extensions */;
             for (int i = 1; i < loaders.length; i++) {
                 if (!loaders[i].getSigning()) {
                     allSigned = false;
@@ -627,10 +670,8 @@ public class JNLPClassLoader extends URLClassLoader {
 
         if (JNLPRuntime.isVerifying()) {
 
-            JarCertVerifier jcv;
-
             try {
-                jcv = verifyJars(initialJars);
+                jcv.add(initialJars, tracker);
             } catch (Exception e) {
                 //we caught an Exception from the JarCertVerifier class.
                 //Note: one of these exceptions could be from not being able
@@ -641,14 +682,13 @@ public class JNLPClassLoader extends URLClassLoader {
             }
 
             //Case when at least one jar has some signing
-            if (jcv.anyJarsSigned() && jcv.isFullySignedByASingleCert()) {
+            if (jcv.isFullySigned()) {
                 signing = true;
 
                 if (!jcv.allJarsSigned() &&
                                     !SecurityDialogs.showNotAllSignedWarningDialog(file))
                     throw new LaunchException(file, null, R("LSFatal"), R("LCClient"), R("LSignedAppJarUsingUnsignedJar"), R("LSignedAppJarUsingUnsignedJarInfo"));
 
-
                 // Check for main class in the downloaded jars, and check/verify signed JNLP fill
                 checkForMain(initialJars);
 
@@ -673,10 +713,10 @@ public class JNLPClassLoader extends URLClassLoader {
                 // If main jar was found, but a signed JNLP file was not located
                 if (!isSignedJNLP && foundMainJar) 
                     file.setSignedJNLPAsMissing();
-                
+
                 //user does not trust this publisher
-                if (!jcv.getAlreadyTrustPublisher()) {
-                    checkTrustWithUser(jcv);
+                if (!jcv.isTriviallySigned()) {
+                    checkTrustWithUser();
                 } else {
                     /**
                      * If the user trusts this publisher (i.e. the publisher's certificate
@@ -685,9 +725,9 @@ public class JNLPClassLoader extends URLClassLoader {
                 }
             } else {
 
+                // Otherwise this jar is simply unsigned -- make sure to ask
+                // for permission on certain actions
                 signing = false;
-                //otherwise this jar is simply unsigned -- make sure to ask
-                //for permission on certain actions
             }
         }
 
@@ -752,7 +792,7 @@ public class JNLPClassLoader extends URLClassLoader {
      * @param jars Jars that are checked to see if they contain the main class
      * @throws LaunchException Thrown if the signed JNLP file, within the main jar, fails to be verified or does not match
      */
-    private void checkForMain(List<JARDesc> jars) throws LaunchException {
+    void checkForMain(List<JARDesc> jars) throws LaunchException {
 
         // Check launch info
         if (mainClass == null) {
@@ -822,6 +862,8 @@ public class JNLPClassLoader extends URLClassLoader {
                         break;
                     }
                 }
+
+                jarFile.close();
             } catch (IOException e) {
                 /*
                  * After this exception is caught, it is escaped. This will skip
@@ -838,18 +880,21 @@ public class JNLPClassLoader extends URLClassLoader {
      * @param location The JAR location
      * @return the main class name, null if there isn't one of if there was an error
      */
-    private String getMainClassName(URL location) {
+    String getMainClassName(URL location) {
 
         String mainClass = null;
         File f = tracker.getCacheFile(location);
 
         if( f != null) {
+            JarFile mainJar = null;
             try {
-                JarFile mainJar = new JarFile(f);
+                mainJar = new JarFile(f);
                 mainClass = mainJar.getManifest().
                         getMainAttributes().getValue("Main-Class");
             } catch (IOException ioe) {
                 mainClass = null;
+            } finally {
+                StreamUtils.closeSilently(mainJar);
             }
         }
 
@@ -887,7 +932,6 @@ public class JNLPClassLoader extends URLClassLoader {
     private void verifySignedJNLP(JARDesc jarDesc, JarFile jarFile)
             throws LaunchException {
 
-        JarCertVerifier signer = new JarCertVerifier();
         List<JARDesc> desc = new ArrayList<JARDesc>();
         desc.add(jarDesc);
 
@@ -898,9 +942,9 @@ public class JNLPClassLoader extends URLClassLoader {
         InputStreamReader jnlpReader = null;
 
         try {
-            signer.verifyJars(desc, tracker);
-
-            if (signer.allJarsSigned()) { // If the jar is signed
+            // NOTE: verification should have happened by now. In other words,
+            // calling jcv.verifyJars(desc, tracker) here should have no affect.
+            if (jcv.isFullySigned()) {
 
                 Enumeration<JarEntry> entries = jarFile.entries();
                 JarEntry je;
@@ -984,58 +1028,34 @@ public class JNLPClassLoader extends URLClassLoader {
             /*
              * After this exception is caught, it is escaped. If an exception is
              * thrown while handling the jar file, (mainly for
-             * JarCertVerifier.verifyJars) it assumes the jar file is unsigned and
+             * JarCertVerifier.add) it assumes the jar file is unsigned and
              * skip the check for a signed JNLP file
              */
             
         } finally {
 
             //Close all streams
-            closeStream(inStream);
-            closeStream(inputReader);
-            closeStream(fr);
-            closeStream(jnlpReader);
+            StreamUtils.closeSilently(inStream);
+            StreamUtils.closeSilently(inputReader);
+            StreamUtils.closeSilently(fr);
+            StreamUtils.closeSilently(jnlpReader);
         }
 
         if (JNLPRuntime.isDebug())
             System.err.println("Ending check for signed JNLP file...");
     }
 
-    /***
-     * Closes a stream
-     * 
-     * @param stream the stream that will be closed
+    /**
+     * Prompt the user for trust on all the signers that require approval.
+     * @throws LaunchException if the user does not approve every dialog prompt.
      */
-    private void closeStream (Closeable stream) {
-        if (stream != null)
-            try {
-                stream.close();
-            } catch (Exception e) {
-                e.printStackTrace(System.err);
-            }
-    }
-    
-    private void checkTrustWithUser(JarCertVerifier jcv) throws LaunchException {
+    private void checkTrustWithUser() throws LaunchException {
         if (JNLPRuntime.isTrustAll()){
             return;
         }
-        if (!jcv.getRootInCacerts()) { //root cert is not in cacerts
-            boolean b = SecurityDialogs.showCertWarningDialog(
-                    AccessType.UNVERIFIED, file, jcv);
-            if (!b)
-                throw new LaunchException(null, null, R("LSFatal"),
-                        R("LCLaunching"), R("LNotVerified"), "");
-        } else if (jcv.getRootInCacerts()) { //root cert is in cacerts
-            boolean b = false;
-            if (jcv.noSigningIssues())
-                b = SecurityDialogs.showCertWarningDialog(
-                        AccessType.VERIFIED, file, jcv);
-            else if (!jcv.noSigningIssues())
-                b = SecurityDialogs.showCertWarningDialog(
-                        AccessType.SIGNING_ERROR, file, jcv);
-            if (!b)
-                throw new LaunchException(null, null, R("LSFatal"),
-                        R("LCLaunching"), R("LCancelOnUserRequest"), "");
+
+        if (jcv.isFullySigned() && !jcv.getAlreadyTrustPublisher()) {
+            jcv.checkTrustWithUser(file);
         }
     }
 
@@ -1140,7 +1160,7 @@ public class JNLPClassLoader extends URLClassLoader {
             }
 
             // Class from host X should be allowed to connect to host X
-            if (cs.getLocation().getHost().length() > 0)
+            if (cs.getLocation() != null && cs.getLocation().getHost().length() > 0)
                 result.add(new SocketPermission(cs.getLocation().getHost(),
                         "connect, accept"));
 
@@ -1249,15 +1269,25 @@ public class JNLPClassLoader extends URLClassLoader {
                                         continue;
                                     }
 
-                                    JarCertVerifier signer = new JarCertVerifier();
-                                    List<JARDesc> jars = new ArrayList<JARDesc>();
-                                    JARDesc jarDesc = new JARDesc(new File(extractedJarLocation).toURL(), null, null, false, false, false, false);
-                                    jars.add(jarDesc);
                                     tracker.addResource(new File(extractedJarLocation).toURL(), null, null, null);
-                                    signer.verifyJars(jars, tracker);
 
-                                    if (signer.anyJarsSigned() && !signer.getAlreadyTrustPublisher()) {
-                                        checkTrustWithUser(signer);
+                                    URL codebase = file.getCodeBase();
+                                    if (codebase == null) {
+                                        //FIXME: codebase should be the codebase of the Main Jar not
+                                        //the location. Although, it still works in the current state.
+                                        codebase = file.getResources().getMainJAR().getLocation();
+                                    }
+
+                                    SecurityDesc jarSecurity = null;
+                                    if (jcv.isFullySigned()) {
+                                        // Already trust application, nested jar should be given
+                                        jarSecurity = new SecurityDesc(file,
+                                                SecurityDesc.ALL_PERMISSIONS,
+                                                codebase.getHost());
+                                    } else {
+                                        jarSecurity = new SecurityDesc(file,
+                                                SecurityDesc.SANDBOX_PERMISSIONS,
+                                                codebase.getHost());
                                     }
 
                                     try {
@@ -1267,25 +1297,6 @@ public class JNLPClassLoader extends URLClassLoader {
                                         CachedJarFileCallback.getInstance().addMapping(fakeRemote, fileURL);
                                         addURL(fakeRemote);
 
-                                        SecurityDesc jarSecurity = file.getSecurity();
-
-                                        if (file instanceof PluginBridge) {
-
-                                            URL codebase = null;
-
-                                            if (file.getCodeBase() != null) {
-                                                codebase = file.getCodeBase();
-                                            } else {
-                                                //Fixme: codebase should be the codebase of the Main Jar not
-                                                //the location. Although, it still works in the current state.
-                                                codebase = file.getResources().getMainJAR().getLocation();
-                                            }
-
-                                            jarSecurity = new SecurityDesc(file,
-                                                    SecurityDesc.ALL_PERMISSIONS,
-                                                    codebase.getHost());
-                                        }
-
                                         jarLocationSecurityMap.put(fakeRemote, jarSecurity);
 
                                     } catch (MalformedURLException mfue) {
@@ -1299,6 +1310,7 @@ public class JNLPClassLoader extends URLClassLoader {
                                 jarEntries.add(je.getName());
                             }
 
+                            jarFile.close();
                         }
 
                         addURL(jar.getLocation());
@@ -1322,6 +1334,8 @@ public class JNLPClassLoader extends URLClassLoader {
                             JarIndex index = JarIndex.getJarIndex(jarFile, null);
                             if (index != null)
                                 jarIndexes.add(index);
+
+                            jarFile.close();
                         } else {
                             CachedJarFileCallback.getInstance().addMapping(jar.getLocation(), jar.getLocation());
                         }
@@ -1395,6 +1409,7 @@ public class JNLPClassLoader extends URLClassLoader {
                                      new FileOutputStream(outFile));
 
             }
+            jarFile.close();
         } catch (IOException ex) {
             if (JNLPRuntime.isDebug())
                 ex.printStackTrace();
@@ -1497,18 +1512,6 @@ public class JNLPClassLoader extends URLClassLoader {
         CacheUtil.waitForResources(app, tracker, urls, file.getTitle());
     }
 
-    /**
-         * Verifies code signing of jars to be used.
-         *
-         * @param jars the jars to be verified.
-         */
-    private JarCertVerifier verifyJars(List<JARDesc> jars) throws Exception {
-
-        jcv = new JarCertVerifier();
-        jcv.verifyJars(jars, tracker);
-        return jcv;
-    }
-
     /**
      * Find the loaded class in this loader or any of its extension loaders.
      */
@@ -1517,17 +1520,7 @@ public class JNLPClassLoader extends URLClassLoader {
             Class result = null;
 
             if (loaders[i] == this) {
-                final String fName = name;
-                try {
-                    result = AccessController.doPrivileged(
-                            new PrivilegedExceptionAction<Class<?>>() {
-                                public Class<?> run() {
-                                    return JNLPClassLoader.super.findLoadedClass(fName);
-                                }
-                            }, getAccessControlContextForClassLoading());
-                } catch (PrivilegedActionException pae) {
-                    result = null;
-                }
+                result = JNLPClassLoader.super.findLoadedClass(name);
             } else {
                 result = loaders[i].findLoadedClassAll(name);
             }
@@ -1645,13 +1638,22 @@ public class JNLPClassLoader extends URLClassLoader {
      * @param desc the JARDesc for the new jar
      */
     private void addNewJar(final JARDesc desc) {
+        this.addNewJar(desc, JNLPRuntime.getDefaultUpdatePolicy());
+    }
+
+    /**
+     * Adds a new JARDesc into this classloader.
+     * @param desc the JARDesc for the new jar
+     * @param updatePolicy the UpdatePolicy for the resource
+     */
+    private void addNewJar(final JARDesc desc, UpdatePolicy updatePolicy) {
 
         available.add(desc);
 
         tracker.addResource(desc.getLocation(),
                 desc.getVersion(),
                 null,
-                JNLPRuntime.getDefaultUpdatePolicy()
+                updatePolicy
                 );
 
         // Give read permissions to the cached jar file
@@ -1675,7 +1677,6 @@ public class JNLPClassLoader extends URLClassLoader {
 
             // Verify if needed
 
-            final JarCertVerifier signer = new JarCertVerifier();
             final List<JARDesc> jars = new ArrayList<JARDesc>();
             jars.add(desc);
 
@@ -1687,14 +1688,12 @@ public class JNLPClassLoader extends URLClassLoader {
 
             AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                 public Void run() throws Exception {
-                    signer.verifyJars(jars, tracker);
+                    jcv.add(jars, tracker);
 
-                    if (signer.anyJarsSigned() && !signer.getAlreadyTrustPublisher()) {
-                        checkTrustWithUser(signer);
-                    }
+                    checkTrustWithUser();
 
                     final SecurityDesc security;
-                    if (signer.anyJarsSigned()) {
+                    if (jcv.isFullySigned()) {
                         security = new SecurityDesc(file,
                                 SecurityDesc.ALL_PERMISSIONS,
                                 file.getCodeBase().getHost());
@@ -1725,6 +1724,7 @@ public class JNLPClassLoader extends URLClassLoader {
     /**
      * Find the class in this loader or any of its extension loaders.
      */
+    @Override
     protected Class findClass(String name) throws ClassNotFoundException {
         for (int i = 0; i < loaders.length; i++) {
             try {
@@ -1741,13 +1741,16 @@ public class JNLPClassLoader extends URLClassLoader {
                 }
             } catch (ClassNotFoundException ex) {
             } catch (ClassFormatError cfe) {
+                cfe.printStackTrace();
             } catch (PrivilegedActionException pae) {
+            } catch (NullJnlpFileException ex) {
+                throw new ClassNotFoundException(this.mainClass + " in main classloader ", ex);
             }
         }
 
         // Try codebase loader
         if (codeBaseLoader != null)
-            return codeBaseLoader.findClass(name, true);
+            return codeBaseLoader.findClassNonRecursive(name);
 
         // All else failed. Throw CNFE
         throw new ClassNotFoundException(name);
@@ -2134,6 +2137,108 @@ public class JNLPClassLoader extends URLClassLoader {
         }
     }
 
+    /**
+     * Returns all loaders that this loader uses, including itself
+     */
+    JNLPClassLoader[] getLoaders() {
+        return loaders;
+    }
+
+    /**
+     * Remove jars from the file system.
+     *
+     * @param jars Jars marked for removal.
+     */
+    void removeJars(JARDesc[] jars) {
+
+        for (JARDesc eachJar : jars) {
+            try {
+                tracker.removeResource(eachJar.getLocation());
+            } catch (Exception e) {
+                if (JNLPRuntime.isDebug()) {
+                    System.err.println(e.getMessage());
+                    System.err.println("Failed to remove resource from tracker, continuing..");
+                }
+            }
+
+            File cachedFile = CacheUtil.getCacheFile(eachJar.getLocation(), null);
+            String directoryUrl = CacheUtil.getCacheParentDirectory(cachedFile.getAbsolutePath());
+
+            File directory = new File(directoryUrl);
+
+            if (JNLPRuntime.isDebug())
+                System.out.println("Deleting cached file: " + cachedFile.getAbsolutePath());
+
+            cachedFile.delete();
+
+            if (JNLPRuntime.isDebug())
+                System.out.println("Deleting cached directory: " + directory.getAbsolutePath());
+
+            directory.delete();
+        }
+    }
+
+    /**
+     * Downloads and initializes jars into this loader.
+     *
+     * @param ref Path of the launch or extension JNLP File containing the
+     * resource. If null, main JNLP's file location will be used instead.
+     * @param part The name of the path.
+     * @throws LaunchException
+     */
+    void initializeNewJarDownload(URL ref, String part, Version version) {
+        JARDesc[] jars = ManageJnlpResources.findJars(this, ref, part, version);
+
+        for (JARDesc eachJar : jars) {
+            if (JNLPRuntime.isDebug())
+                System.out.println("Downloading and initializing jar: " + eachJar.getLocation().toString());
+
+            this.addNewJar(eachJar, UpdatePolicy.FORCE);
+        }
+    }
+
+    /**
+     * Manages DownloadService jars which are not mentioned in the JNLP file
+     * @param ref Path to the resource.
+     * @param version The version of resource. If null, no version is specified.
+     * @param action The action to perform with the resource. Either DOWNLOADTOCACHE, REMOVEFROMCACHE, or CHECKCACHE.
+     * @return true if CHECKCACHE and the resource is cached.
+     */
+    boolean manageExternalJars(URL ref, String version, DownloadAction action) {
+        boolean approved = false;
+        JNLPClassLoader foundLoader = LocateJnlpClassLoader.getLoaderByResourceUrl(this, ref, version);
+        Version resourceVersion = (version == null) ? null : new Version(version);
+
+        if (foundLoader != null)
+            approved = true;
+
+        else if (ref.toString().startsWith(file.getCodeBase().toString()))
+            approved = true;
+        else if (SecurityDesc.ALL_PERMISSIONS.equals(security.getSecurityType()))
+            approved = true;
+
+        if (approved) {
+            if (foundLoader == null)
+                foundLoader = this;
+
+            if (action == DownloadAction.DOWNLOAD_TO_CACHE) {
+                JARDesc jarToCache = new JARDesc(ref, resourceVersion, null, false, true, false, true);
+                if (JNLPRuntime.isDebug())
+                    System.out.println("Downloading and initializing jar: " + ref.toString());
+
+                foundLoader.addNewJar(jarToCache, UpdatePolicy.FORCE);
+
+            } else if (action == DownloadAction.REMOVE_FROM_CACHE) {
+                JARDesc[] jarToRemove = { new JARDesc(ref, resourceVersion, null, false, true, false, true) };
+                foundLoader.removeJars(jarToRemove);
+
+            } else if (action == DownloadAction.CHECK_CACHE) {
+                return CacheUtil.isCached(ref, resourceVersion);
+            }
+        }
+        return false;
+    }
+
     /**
      * Decrements loader use count by 1
      * 
@@ -2180,8 +2285,6 @@ public class JNLPClassLoader extends URLClassLoader {
             return context; // If context already has all permissions, don't bother
         } catch (AccessControlException ace) {
             // continue below
-        } catch (ClassCircularityError cce) {
-            // continue below
         }
 
         // Since this is for class-loading, technically any class from one jar
@@ -2227,7 +2330,7 @@ public class JNLPClassLoader extends URLClassLoader {
         ConcurrentHashMap<String, URL[]> notFoundResources = new ConcurrentHashMap<String, URL[]>();
 
         public CodeBaseClassLoader(URL[] urls, JNLPClassLoader cl) {
-            super(urls);
+            super(urls, cl);
             parentJNLPClassLoader = cl;
         }
 
@@ -2236,21 +2339,7 @@ public class JNLPClassLoader extends URLClassLoader {
             super.addURL(url); 
         }
 
-        @Override
-        public Class<?> findClass(String name) throws ClassNotFoundException {
-            return findClass(name, false);
-        }
-
-        public Class<?> findClass(String name, boolean recursivelyInvoked) throws ClassNotFoundException {
-
-            if (!recursivelyInvoked) {
-                try {
-                    return parentJNLPClassLoader.findClass(name);
-                } catch (ClassNotFoundException cnfe) {
-                    // continue
-                }
-            }
-
+        Class<?> findClassNonRecursive(String name) throws ClassNotFoundException {
             // If we have searched this path before, don't try again
             if (Arrays.equals(super.getURLs(), notFoundResources.get(name)))
                 throw new ClassNotFoundException(name);
@@ -2265,10 +2354,19 @@ public class JNLPClassLoader extends URLClassLoader {
                         }, parentJNLPClassLoader.getAccessControlContextForClassLoading());
             } catch (PrivilegedActionException pae) {
                 notFoundResources.put(name, super.getURLs());
-                throw new ClassNotFoundException("Could not find class " + name);
+                throw new ClassNotFoundException("Could not find class " + name, pae);
+            } catch (NullJnlpFileException njf) {
+                notFoundResources.put(name, super.getURLs());
+                throw new ClassNotFoundException("Could not find class " + name, njf);
             }
         }
 
+        @Override
+        public Class<?> findClass(String name) throws ClassNotFoundException {
+            // Calls JNLPClassLoader#findClass which may call into this.findClassNonRecursive
+            return getParentJNLPClassLoader().findClass(name);
+        }
+
         /**
          * Returns the output of super.findLoadedClass().
          * 
@@ -2329,7 +2427,7 @@ public class JNLPClassLoader extends URLClassLoader {
                                 }
                             }, parentJNLPClassLoader.getAccessControlContextForClassLoading());
                 } catch (PrivilegedActionException pae) {
-                }
+                } 
 
                 if (url == null) {
                     notFoundResources.put(name, super.getURLs());
@@ -2341,4 +2439,6 @@ public class JNLPClassLoader extends URLClassLoader {
             return null;
         }
     }
+
+
 }
