@@ -38,6 +38,7 @@ exception statement from your version.
 package net.sourceforge.jnlp.security;
 
 import net.sourceforge.jnlp.JNLPFile;
+import net.sourceforge.jnlp.PluginBridge;
 import net.sourceforge.jnlp.runtime.JNLPRuntime;
 import net.sourceforge.jnlp.security.SecurityDialogs.AccessType;
 import net.sourceforge.jnlp.security.SecurityDialogs.DialogType;
@@ -52,6 +53,7 @@ import java.security.cert.X509Certificate;
 import java.util.concurrent.CopyOnWriteArrayList;
 
 import java.util.List;
+import net.sourceforge.jnlp.util.ScreenFinder;
 
 /**
  * Provides methods for showing security warning dialogs for a wide range of
@@ -305,6 +307,8 @@ public class SecurityDialog extends JDialog {
             panel = new AppletWarningPane(this, this.certVerifier);
         else if (dialogType == DialogType.NOTALLSIGNED_WARNING)
             panel = new NotAllSignedWarningPane(this);
+        else if (dialogType == DialogType.UNSIGNED_WARNING) // Only necessary for applets on 'high security' or above
+            panel = new UnsignedAppletTrustWarningDialog(this, (PluginBridge)file);
         else if (dialogType == DialogType.AUTHENTICATION)
             panel = new PasswordAuthenticationPane(this, extras);
 
@@ -312,11 +316,7 @@ public class SecurityDialog extends JDialog {
     }
 
     private static void centerDialog(JDialog dialog) {
-        Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
-        Dimension dialogSize = dialog.getSize();
-
-        dialog.setLocation((screen.width - dialogSize.width) / 2,
-                        (screen.height - dialogSize.height) / 2);
+        ScreenFinder.centerWindowsToCurrentScreen(dialog);
     }
 
     private void selectDefaultButton() {
