@@ -56,15 +56,8 @@ exception statement from your version. */
 #include <vector>
 
 #include <npapi.h>
-
-#if MOZILLA_VERSION_COLLAPSED < 1090100
-#include <npupp.h>
-#else
-#include <npapi.h>
+#include <glib.h>
 #include <npruntime.h>
-#endif
-
-#include "IcedTeaNPPlugin.h"
 
 #define PLUGIN_DEBUG(...) \
   do                                                          \
@@ -212,6 +205,12 @@ class IcedTeaPluginUtilities
     	/* Copies a variant data type into a C++ string */
     	static std::string NPVariantAsString(NPVariant variant);
 
+        /* This must be freed with browserfunctions.memfree */
+        static NPString NPStringCopy(const std::string& result);
+
+        /* This must be freed with browserfunctions.releasevariantvalue */
+        static NPVariant NPVariantStringCopy(const std::string& result);
+
     	/* Frees the given vector and the strings that its contents point to */
     	static void freeStringPtrVector(std::vector<std::string*>* v);
 
@@ -243,7 +242,7 @@ class IcedTeaPluginUtilities
 
     	static void printNPVariant(NPVariant variant);
 
-    	static void NPVariantToString(NPVariant variant, std::string* result);
+        static void NPVariantToString(NPVariant variant, std::string* result);
 
         static bool javaResultToNPVariant(NPP instance,
                                           std::string* java_result,
@@ -274,6 +273,11 @@ class IcedTeaPluginUtilities
 
     	/* Posts call in async queue and waits till execution completes */
     	static void callAndWaitForResult(NPP instance, void (*func) (void *), AsyncCallThreadData* data);
+
+        /*cutting whitespaces from end and start of string*/
+        static void trim(std::string& str);
+        static bool file_exists(std::string filename);
+
 };
 
 /*
@@ -340,4 +344,6 @@ class MessageBus
         void post(const char* message);
 };
 
+
+
 #endif // __ICEDTEAPLUGINUTILS_H__
