@@ -197,7 +197,7 @@ IcedTeaScriptableJavaPackageObject::getPackageName()
 void
 IcedTeaScriptableJavaPackageObject::deAllocate(NPObject *npobj)
 {
-    browser_functions.releaseobject(npobj);
+    delete (IcedTeaScriptableJavaPackageObject*)npobj;
 }
 
 void
@@ -489,7 +489,7 @@ IcedTeaScriptableJavaObject::setIsArray(bool isArray)
 void
 IcedTeaScriptableJavaObject::deAllocate(NPObject *npobj)
 {
-	browser_functions.releaseobject(npobj);
+    delete (IcedTeaScriptableJavaObject*)npobj;
 }
 
 void
@@ -591,7 +591,10 @@ IcedTeaScriptableJavaObject::invoke(NPObject *npobj, NPIdentifier name, const NP
 
     if (java_result->error_occurred)
     {
-        browser_functions.setexception(npobj, java_result->error_msg->c_str());
+        // error message must be allocated on heap
+        char* error_msg = (char*) malloc(java_result->error_msg->length()*sizeof(char));
+        strcpy(error_msg, java_result->error_msg->c_str());
+        browser_functions.setexception(npobj, error_msg);
         return false;
     }
 
@@ -850,7 +853,11 @@ IcedTeaScriptableJavaObject::construct(NPObject *npobj, const NPVariant *args, u
         createJavaObjectFromVariant(instance, args[i], &id);
         if (id == "0")
         {
-            browser_functions.setexception(npobj, "Unable to create argument on Java side");
+            // error message must be allocated on heap
+            char* error_msg = (char*) malloc(1024*sizeof(char));
+            strcpy(error_msg, "Unable to create argument on Java side");
+
+            browser_functions.setexception(npobj, error_msg);
             return false;
         }
 
@@ -864,7 +871,12 @@ IcedTeaScriptableJavaObject::construct(NPObject *npobj, const NPVariant *args, u
 
     if (java_result->error_occurred)
     {
-        browser_functions.setexception(npobj, java_result->error_msg->c_str());
+        // error message must be allocated on heap
+        int length = java_result->error_msg->length();
+        char* error_msg = (char*) malloc((length+1)*sizeof(char));
+        strcpy(error_msg, java_result->error_msg->c_str());
+
+        browser_functions.setexception(npobj, error_msg);
         return false;
     }
 
