@@ -53,13 +53,16 @@ import java.security.Permissions;
 import java.security.PrivilegedAction;
 import java.security.ProtectionDomain;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Hashtable;
 import java.util.List;
 import java.util.Map;
 
-import net.sourceforge.jnlp.runtime.JNLPRuntime;
 import net.sourceforge.jnlp.DefaultLaunchHandler;
+import net.sourceforge.jnlp.runtime.JNLPRuntime;
+import netscape.javascript.JSObject;
 import netscape.javascript.JSObjectCreatePermission;
+import netscape.javascript.JSUtil;
 
 class Signature {
     private String signature;
@@ -235,18 +238,23 @@ public class PluginAppletSecurityContext {
 
     long startTime = 0;
 
-    public PluginAppletSecurityContext(int identifier) {
+    /* Package-private constructor that allows for bypassing security manager installation.
+     * This is useful for testing. Note that while the public constructor should be used otherwise,
+     * the security installation can't be bypassed if it has already occurred.*/
+    PluginAppletSecurityContext(int identifier, boolean ensureSecurityContext) {
         this.identifier = identifier;
 
-        // We need a security manager.. and since there is a good chance that
-        // an applet will be loaded at some point, we should make it the SM
-        // that JNLPRuntime will try to install
-        if (System.getSecurityManager() == null) {
-            JNLPRuntime.initialize(/* isApplication */false);
-            JNLPRuntime.setDefaultLaunchHandler(new DefaultLaunchHandler(System.err));
-        }
+        if (ensureSecurityContext) {
+            // We need a security manager.. and since there is a good chance that
+            // an applet will be loaded at some point, we should make it the SM
+            // that JNLPRuntime will try to install
+            if (System.getSecurityManager() == null) {
+                JNLPRuntime.initialize(/* isApplication */false);
+                JNLPRuntime.setDefaultLaunchHandler(new DefaultLaunchHandler(System.err));
+            }
 
-        JNLPRuntime.disableExit();
+            JNLPRuntime.disableExit();
+        }
 
         URL u = null;
         try {
@@ -258,6 +266,10 @@ public class PluginAppletSecurityContext {
         this.classLoaders.put(liveconnectLoader, u);
     }
 
+    public PluginAppletSecurityContext(int identifier) {
+        this(identifier, true);
+    }
+
     private static <V> V parseCall(String s, ClassLoader cl, Class<V> c) {
         if (c == Integer.class)
             return c.cast(new Integer(s));
@@ -318,6 +330,83 @@ public class PluginAppletSecurityContext {
         return map;
     }
 
+    private static long privilegedJSObjectUnbox(final JSObject js) {
+        return AccessController.doPrivileged(new PrivilegedAction<Long>() {
+            public Long run() {  
+                return JSUtil.getJSObjectInternalReference(js); 
+            }
+        });
+    }
+
+    /**
+     * Create a string that identifies a Java object precisely, for passing to 
+     * Javascript.
+     * 
+     * For builtin value types, a 'literalreturn' prefix is used and the object 
+     * is passed with a string representation.
+     * 
+     * For JSObject's, a 'jsobject' prefix is used and the object is passed 
+     * with the JSObject's internal identifier.
+     * 
+     * For other Java objects, an object store reference is used.
+     * 
+     * @param obj the object for which to create an identifier
+     * @param type the type to use for representation decisions
+     * @param unboxPrimitives whether to treat boxed primitives as value types
+     * @return an identifier string
+     */
+    public String toObjectIDString(Object obj, Class<?> type, boolean unboxPrimitives) {
+
+        /* Void (can occur from declared return type), pass special "void" string: */
+        if (type == Void.TYPE) {
+            return "literalreturn void";
+        }
+
+        /* Null, pass special "null" string: */
+        if (obj == null) {
+            return "literalreturn null";
+        }
+
+        /* Primitive, accurately represented by its toString() form: */
+        boolean returnAsString = ( type == Boolean.TYPE
+                                || type == Byte.TYPE 
+                                || type == Short.TYPE 
+                                || type == Integer.TYPE 
+                                || type == Long.TYPE );
+        if (unboxPrimitives) {
+            returnAsString = ( returnAsString 
+                            || type == Boolean.class
+                            || type == Byte.class
+                            || type == Short.class
+                            || type == Integer.class 
+                            || type == Long.class);
+        }
+        if (returnAsString) {
+            return "literalreturn " + obj.toString();
+        } 
+
+        /* Floating point number, we ensure we give enough precision: */
+        if ( type == Float.TYPE || type == Double.TYPE || 
+                ( unboxPrimitives && (type == Float.class || type == Double.class) )) {
+            return "literalreturn " + String.format("%308.308e", obj);
+        }
+
+        /* Character that should be returned as number: */
+        if (type == Character.TYPE || (unboxPrimitives && type == Character.class)) {
+            return "literalreturn " + (int) (Character) obj;
+        }
+
+        /* JSObject, unwrap underlying Javascript reference: */
+        if (type == netscape.javascript.JSObject.class) {
+            long reference = privilegedJSObjectUnbox((JSObject)obj);
+            return "jsobject " + Long.toString(reference);
+        }
+
+        /* Other kind of object, track this object and return our reference: */
+        store.reference(obj);
+        return store.getIdentifier(obj).toString();
+    }
+
     public void handleMessage(int reference, String src, AccessControlContext callContext, String message) {
 
         startTime = new java.util.Date().getTime();
@@ -426,56 +515,16 @@ public class PluginAppletSecurityContext {
                 if (ret instanceof Throwable)
                     throw (Throwable) ret;
 
-                if (ret == null) {
-                    write(reference, "GetStaticField literalreturn null");
-                } else if (f.getType() == Boolean.TYPE
-                        || f.getType() == Byte.TYPE
-                        || f.getType() == Short.TYPE
-                        || f.getType() == Integer.TYPE
-                        || f.getType() == Long.TYPE) {
-                    write(reference, "GetStaticField literalreturn " + ret);
-                } else if (f.getType() == Float.TYPE
-                                || f.getType() == Double.TYPE) {
-                    write(reference, "GetStaticField literalreturn " + String.format("%308.308e", ret));
-                } else if (f.getType() == Character.TYPE) {
-                    write(reference, "GetStaticField literalreturn " + (int) (Character) ret);
-                } else {
-                    // Track returned object.
-                    store.reference(ret);
-                    write(reference, "GetStaticField " + store.getIdentifier(ret));
-                }
+                String objIDStr = toObjectIDString(ret, f.getType(), false /*do not unbox primitives*/);
+                write(reference, "GetStaticField " + objIDStr);
             } else if (message.startsWith("GetValue")) {
                 String[] args = message.split(" ");
                 Integer index = parseCall(args[1], null, Integer.class);
 
                 Object ret = store.getObject(index);
 
-                if (ret == null) {
-                    write(reference, "GetValue literalreturn null");
-                } else if (ret.getClass() == Boolean.TYPE
-                        || ret.getClass() == Boolean.class
-                        || ret.getClass() == Byte.TYPE
-                        || ret.getClass() == Byte.class
-                        || ret.getClass() == Short.TYPE
-                        || ret.getClass() == Short.class
-                        || ret.getClass() == Integer.TYPE
-                        || ret.getClass() == Integer.class
-                        || ret.getClass() == Long.TYPE
-                        || ret.getClass() == Long.class) {
-                    write(reference, "GetValue literalreturn " + ret);
-                } else if (ret.getClass() == Float.TYPE
-                        || ret.getClass() == Float.class
-                        || ret.getClass() == Double.TYPE
-                        || ret.getClass() == Double.class) {
-                    write(reference, "GetValue literalreturn " + String.format("%308.308e", ret));
-                } else if (ret.getClass() == Character.TYPE
-                        || ret.getClass() == Character.class) {
-                    write(reference, "GetValue literalreturn " + (int) (Character) ret);
-                } else {
-                    // Track returned object.
-                    store.reference(ret);
-                    write(reference, "GetValue " + store.getIdentifier(ret));
-                }
+                String objIDStr = toObjectIDString(ret, ret.getClass(), true /*unbox primitives*/);
+                write(reference, "GetValue " + objIDStr);
             } else if (message.startsWith("SetStaticField") ||
                                    message.startsWith("SetField")) {
                 String[] args = message.split(" ");
@@ -486,12 +535,10 @@ public class PluginAppletSecurityContext {
                 final Object o = store.getObject(classOrObjectID);
                 final Field f = (Field) store.getObject(fieldID);
 
-                final Object fValue = MethodOverloadResolver.getCostAndCastedObject(value, f.getType())[1];
+                final Object fValue = MethodOverloadResolver.getCostAndCastedObject(value, f.getType()).getCastedObject();
 
                 AccessControlContext acc = callContext != null ? callContext : getClosedAccessControlContext();
-                checkPermission(src,
-                                                message.startsWith("SetStaticField") ? (Class) o : o.getClass(),
-                                                acc);
+                checkPermission(src, message.startsWith("SetStaticField") ? (Class) o : o.getClass(), acc);
 
                 Object ret = AccessController.doPrivileged(new PrivilegedAction<Object>() {
                     public Object run() {
@@ -514,27 +561,12 @@ public class PluginAppletSecurityContext {
                 Integer arrayID = parseCall(args[1], null, Integer.class);
                 Integer index = parseCall(args[2], null, Integer.class);
 
-                Object ret = Array.get(store.getObject(arrayID), index);
-                Class retClass = store.getObject(arrayID).getClass().getComponentType(); // prevent auto-boxing influence
+                Object array = store.getObject(arrayID);
+                Object ret = Array.get(array, index);
+                Class<?> retClass = array.getClass().getComponentType(); // prevent auto-boxing influence
 
-                if (ret == null) {
-                    write(reference, "GetObjectArrayElement literalreturn null");
-                } else if (retClass == Boolean.TYPE
-                        || retClass == Byte.TYPE
-                        || retClass == Short.TYPE
-                        || retClass == Integer.TYPE
-                        || retClass == Long.TYPE) {
-                    write(reference, "GetObjectArrayElement literalreturn " + ret);
-                } else if (retClass == Float.TYPE
-                                || retClass == Double.TYPE) {
-                    write(reference, "GetObjectArrayElement literalreturn " + String.format("%308.308e", ret));
-                } else if (retClass == Character.TYPE) {
-                    write(reference, "GetObjectArrayElement literalreturn " + (int) (Character) ret);
-                } else {
-                    // Track returned object.
-                    store.reference(ret);
-                    write(reference, "GetObjectArrayElement " + store.getIdentifier(ret));
-                }
+                String objIDStr = toObjectIDString(ret, retClass, false /*do not unbox primitives*/);
+                write(reference, "GetObjectArrayElement " + objIDStr);
 
             } else if (message.startsWith("SetObjectArrayElement")) {
                 String[] args = message.split(" ");
@@ -545,7 +577,7 @@ public class PluginAppletSecurityContext {
                 Object value = store.getObject(objectID);
 
                 // Cast the object to appropriate type before insertion
-                value = MethodOverloadResolver.getCostAndCastedObject(value, store.getObject(arrayID).getClass().getComponentType())[1];
+                value = MethodOverloadResolver.getCostAndCastedObject(value, store.getObject(arrayID).getClass().getComponentType()).getCastedObject();
 
                 Array.set(store.getObject(arrayID), index, value);
 
@@ -584,25 +616,8 @@ public class PluginAppletSecurityContext {
                 if (ret instanceof Throwable)
                     throw (Throwable) ret;
 
-                if (ret == null) {
-                    write(reference, "GetField literalreturn null");
-                } else if (f.getType() == Boolean.TYPE
-                        || f.getType() == Byte.TYPE
-                        || f.getType() == Short.TYPE
-                        || f.getType() == Integer.TYPE
-                        || f.getType() == Long.TYPE) {
-                    write(reference, "GetField literalreturn " + ret);
-                } else if (f.getType() == Float.TYPE
-                                || f.getType() == Double.TYPE) {
-                    write(reference, "GetField literalreturn " + String.format("%308.308e", ret));
-                } else if (f.getType() == Character.TYPE) {
-                    write(reference, "GetField literalreturn " + (int) (Character) ret);
-                } else {
-                    // Track returned object.
-                    store.reference(ret);
-                    write(reference, "GetField " + store.getIdentifier(ret));
-                }
-
+                String objIDStr = toObjectIDString(ret, f.getType(), false /*do not unbox primitives*/);
+                write(reference, "GetField " + objIDStr);
             } else if (message.startsWith("GetObjectClass")) {
                 int oid = Integer.parseInt(message.substring("GetObjectClass"
                                                 .length() + 1));
@@ -625,36 +640,26 @@ public class PluginAppletSecurityContext {
                     c = (Class<?>) store.getObject(objectID);
                 }
 
-                // length -3 to discard first 3, + 2 for holding object
-                // and method name
-                Object[] arguments = new Object[args.length - 1];
-                arguments[0] = c;
-                arguments[1] = methodName;
-                for (int i = 0; i < args.length - 3; i++) {
-                    arguments[i + 2] = store.getObject(parseCall(args[3 + i], null, Integer.class));
-                    PluginDebug.debug("GOT ARG: ", arguments[i + 2]);
+                // Discard first 3 parts of message
+                Object[] arguments = new Object[args.length - 3];
+                for (int i = 0; i < arguments.length; i++) {
+                    arguments[i] = store.getObject(parseCall(args[3 + i], null, Integer.class));
+                    PluginDebug.debug("GOT ARG: ", arguments[i]);
                 }
 
-                Object[] matchingMethodAndArgs = MethodOverloadResolver.getMatchingMethod(arguments);
+                MethodOverloadResolver.ResolvedMethod rm = 
+                        MethodOverloadResolver.getBestMatchMethod(c, methodName, arguments);
 
-                if (matchingMethodAndArgs == null) {
+                if (rm == null) {
                     write(reference, "Error: No suitable method named " + methodName + " with matching args found");
                     return;
                 }
 
-                final Method m = (Method) matchingMethodAndArgs[0];
-                Object[] castedArgs = new Object[matchingMethodAndArgs.length - 1];
-                for (int i = 0; i < castedArgs.length; i++) {
-                    castedArgs[i] = matchingMethodAndArgs[i + 1];
-                }
-
-                String collapsedArgs = "";
-                for (Object arg : castedArgs) {
-                    collapsedArgs += " " + arg;
-                }
+                final Method m = rm.getMethod();
+                final Object[] castedArgs = rm.getCastedParameters();
 
                 PluginDebug.debug("Calling method ", m, " on object ", o
-                                                , " (", c, ") with ", collapsedArgs);
+                                                , " (", c, ") with ", Arrays.toString(castedArgs));
 
                 AccessControlContext acc = callContext != null ? callContext : getClosedAccessControlContext();
                 checkPermission(src, c, acc);
@@ -684,31 +689,12 @@ public class PluginAppletSecurityContext {
                     retO = m.getReturnType().toString();
                 }
 
-                PluginDebug.debug("Calling ", m, " on ", o, " with "
-                                                , collapsedArgs, " and that returned: ", ret
-                                                , " of type ", retO);
+                PluginDebug.debug("Calling ", m, " on ", o, " with ", 
+                        Arrays.toString(castedArgs), " and that returned: ", ret,
+                        " of type ", retO);
 
-                if (m.getReturnType().equals(java.lang.Void.class) ||
-                                    m.getReturnType().equals(java.lang.Void.TYPE)) {
-                    write(reference, "CallMethod literalreturn void");
-                } else if (ret == null) {
-                    write(reference, "CallMethod literalreturn null");
-                } else if (m.getReturnType() == Boolean.TYPE
-                                                || m.getReturnType() == Byte.TYPE
-                                                || m.getReturnType() == Short.TYPE
-                                                || m.getReturnType() == Integer.TYPE
-                                                || m.getReturnType() == Long.TYPE) {
-                    write(reference, "CallMethod literalreturn " + ret);
-                } else if (m.getReturnType() == Float.TYPE
-                                || m.getReturnType() == Double.TYPE) {
-                    write(reference, "CallMethod literalreturn " + String.format("%308.308e", ret));
-                } else if (m.getReturnType() == Character.TYPE) {
-                    write(reference, "CallMethod literalreturn " + (int) (Character) ret);
-                } else {
-                    // Track returned object.
-                    store.reference(ret);
-                    write(reference, "CallMethod " + store.getIdentifier(ret));
-                }
+                String objIDStr = toObjectIDString(ret, m.getReturnType(), false /*do not unbox primitives*/);
+                write(reference, "CallMethod " + objIDStr);
             } else if (message.startsWith("GetSuperclass")) {
                 String[] args = message.split(" ");
                 Integer classID = parseCall(args[1], null, Integer.class);
@@ -776,10 +762,7 @@ public class PluginAppletSecurityContext {
                 buf = new StringBuffer(b.length * 2);
                 buf.append(b.length);
                 for (int i = 0; i < b.length; i++)
-                    buf
-                                                        .append(" "
-                                                                        + Integer
-                                                                                        .toString(((int) b[i]) & 0x0ff, 16));
+                    buf.append(" " + Integer.toString(((int) b[i]) & 0x0ff, 16));
 
                 write(reference, "GetStringUTFChars " + buf);
             } else if (message.startsWith("GetStringChars")) {
@@ -795,10 +778,7 @@ public class PluginAppletSecurityContext {
                 buf = new StringBuffer(b.length * 2);
                 buf.append(b.length);
                 for (int i = 0; i < b.length; i++)
-                    buf
-                                                        .append(" "
-                                                                        + Integer
-                                                                                        .toString(((int) b[i]) & 0x0ff, 16));
+                    buf.append(" " + Integer.toString(((int) b[i]) & 0x0ff, 16));
 
                 PluginDebug.debug("Java: GetStringChars: ", o);
                 PluginDebug.debug("  String BYTES: ", buf);
@@ -815,10 +795,7 @@ public class PluginAppletSecurityContext {
                 buf = new StringBuffer(b.length * 2);
                 buf.append(b.length);
                 for (int i = 0; i < b.length; i++)
-                    buf
-                            .append(" "
-                                    + Integer
-                                            .toString(((int) b[i]) & 0x0ff, 16));
+                    buf.append(" " + Integer.toString(((int) b[i]) & 0x0ff, 16));
 
                 write(reference, "GetToStringValue " + buf);
             } else if (message.startsWith("NewArray")) {
@@ -957,42 +934,30 @@ public class PluginAppletSecurityContext {
             } else if (message.startsWith("NewObject")) {
                 String[] args = message.split(" ");
                 Integer classID = parseCall(args[1], null, Integer.class);
-                Class c = (Class) store.getObject(classID);
-                final Constructor cons;
-                final Object[] fArguments;
+                Class<?> c = (Class<?>) store.getObject(classID);
 
-                Object[] arguments = new Object[args.length - 1];
-                arguments[0] = c;
-                for (int i = 0; i < args.length - 2; i++) {
-                    arguments[i + 1] = store.getObject(parseCall(args[2 + i],
+                // Discard first 2 parts of message
+                Object[] arguments = new Object[args.length - 2];
+                for (int i = 0; i < arguments.length; i++) {
+                    arguments[i] = store.getObject(parseCall(args[2 + i],
                             null, Integer.class));
-                    PluginDebug.debug("GOT ARG: ", arguments[i + 1]);
+                    PluginDebug.debug("GOT ARG: ", arguments[i]);
                 }
 
-                Object[] matchingConstructorAndArgs = MethodOverloadResolver
-                        .getMatchingConstructor(arguments);
+                MethodOverloadResolver.ResolvedMethod resolvedConstructor = 
+                        MethodOverloadResolver.getBestMatchConstructor(c, arguments);
 
-                if (matchingConstructorAndArgs == null) {
+                if (resolvedConstructor == null) {
                     write(reference,
                             "Error: No suitable constructor with matching args found");
                     return;
                 }
 
-                Object[] castedArgs = new Object[matchingConstructorAndArgs.length - 1];
-                for (int i = 0; i < castedArgs.length; i++) {
-                    castedArgs[i] = matchingConstructorAndArgs[i + 1];
-                }
-
-                cons = (Constructor) matchingConstructorAndArgs[0];
-                fArguments = castedArgs;
-
-                String collapsedArgs = "";
-                for (Object arg : fArguments) {
-                    collapsedArgs += " " + arg.toString();
-                }
+                final Constructor<?> cons = resolvedConstructor.getConstructor();
+                final Object[] castedArgs = resolvedConstructor.getCastedParameters();
 
                 PluginDebug.debug("Calling constructor on class ", c,
-                                   " with ", collapsedArgs);
+                                   " with ", Arrays.toString(castedArgs));
 
                 AccessControlContext acc = callContext != null ? callContext : getClosedAccessControlContext();
                 checkPermission(src, c, acc);
@@ -1000,7 +965,7 @@ public class PluginAppletSecurityContext {
                 Object ret = AccessController.doPrivileged(new PrivilegedAction<Object>() {
                     public Object run() {
                         try {
-                            return cons.newInstance(fArguments);
+                            return cons.newInstance(castedArgs);
                         } catch (Throwable t) {
                             return t;
                         }
