@@ -1,5 +1,5 @@
 /* ServerAccess.java
-Copyright (C) 2011 Red Hat, Inc.
+Copyright (C) 2011, 2012 Red Hat, Inc.
 
 This file is part of IcedTea.
 
@@ -58,6 +58,8 @@ import java.util.List;
 import net.sourceforge.jnlp.browsertesting.Browser;
 import net.sourceforge.jnlp.browsertesting.BrowserFactory;
 import net.sourceforge.jnlp.browsertesting.Browsers;
+import net.sourceforge.jnlp.closinglisteners.AutoErrorClosingListener;
+import net.sourceforge.jnlp.closinglisteners.AutoOkClosingListener;
 import org.junit.Assert;
 
 /**
@@ -77,6 +79,11 @@ import org.junit.Assert;
  */
 public class ServerAccess {
 
+    public static enum AutoClose {
+
+        CLOSE_ON_EXCEPTION, CLOSE_ON_CORRECT_END, CLOSE_ON_BOTH
+    }
+
     public static final long NANO_TIME_DELIMITER=1000000l;
     /**
      * java property which value containing path to default (makefile by) directory with deployed resources
@@ -131,8 +138,10 @@ public class ServerAccess {
             System.exit(0);
         } else {
             int port = 44321;
-            if (args.length > 0) {
-                port=new Integer(args[0]);
+            if (args.length > 0 && args[0].equalsIgnoreCase("randomport")) {
+                port = findFreePort();
+            } else if (args.length > 0) {
+                port = new Integer(args[0]);
             }
             getIndependentInstance(port);
             while (true) {
@@ -156,7 +165,7 @@ public class ServerAccess {
         return port;
     }
     public static final  String HEADLES_OPTION="-headless";
-    public static final  String VERBOSE_OPTION="-verbose ";
+    public static final  String VERBOSE_OPTION="-verbose";
 
     /**
      * we would like to have an singleton instance ASAP
@@ -545,6 +554,20 @@ public class ServerAccess {
     public ProcessResult executeJavaws(String resource,ContentReaderListener stdoutl,ContentReaderListener stderrl) throws Exception {
         return executeJavaws(null, resource,stdoutl,stderrl);
     }
+
+    public net.sourceforge.jnlp.ProcessResult executeBrowser(String string, AutoClose autoClose) throws Exception {
+        ClosingListener errClosing = null;
+        ClosingListener outClosing = null;
+        if (autoClose == AutoClose.CLOSE_ON_BOTH || autoClose == AutoClose.CLOSE_ON_EXCEPTION){
+            errClosing=new AutoErrorClosingListener();
+        }
+        if (autoClose == AutoClose.CLOSE_ON_BOTH || autoClose == AutoClose.CLOSE_ON_CORRECT_END){
+            outClosing=new AutoOkClosingListener();
+        }
+        return executeBrowser(string, outClosing, errClosing);
+    }
+    
+
     public ProcessResult executeBrowser(String resource) throws Exception {
         return executeBrowser(getBrowserParams(), resource);
     }
@@ -552,6 +575,10 @@ public class ServerAccess {
         return executeBrowser(getBrowserParams(), resource, stdoutl, stderrl);
     }
 
+    public ProcessResult executeBrowser(String resource, List<ContentReaderListener> stdoutl, List<ContentReaderListener> stderrl) throws Exception {
+        return executeBrowser(getBrowserParams(), resource, stdoutl, stderrl);
+    }
+
     /**
      *  wrapping method to executeProcess (eg: javaws arg arg http://localhost:port/resource)
      * will execute default javaws (@see JAVAWS_BUILD_BIN) upon default url upon cached server (@see SERVER_NAME @see getPort(), @see getInstance()))
@@ -571,28 +598,54 @@ public class ServerAccess {
     }
 
     public ProcessResult executeBrowser(List<String> otherargs, String resource) throws Exception {
-        return executeProcessUponURL(getBrowserLocation(), otherargs, getUrlUponThisInstance(resource));
+        return executeBrowser(otherargs, getUrlUponThisInstance(resource));        
     }
-    public ProcessResult executeBrowser(List<String> otherargs, String resource,ContentReaderListener stdoutl,ContentReaderListener stderrl) throws Exception {
-        return executeProcessUponURL(getBrowserLocation(), otherargs, getUrlUponThisInstance(resource),stdoutl,stderrl);
+    
+     public ProcessResult executeBrowser(List<String> otherargs, URL url) throws Exception {
+        ProcessWrapper rpw = new ProcessWrapper(getBrowserLocation(), otherargs, url);
+        rpw.setReactingProcess(getCurrentBrowser());//current browser may be null, but it does not metter
+        return rpw.execute();
     }
 
-     public ProcessResult executeBrowser(Browser b,List<String> otherargs, String resource) throws Exception {
-        return executeProcessUponURL(b.getBin(), otherargs, getUrlUponThisInstance(resource));
+    public ProcessResult executeBrowser(List<String> otherargs, String resource, ContentReaderListener stdoutl, ContentReaderListener stderrl) throws Exception {
+        ProcessWrapper rpw = new ProcessWrapper(getBrowserLocation(), otherargs, getUrlUponThisInstance(resource), stdoutl, stderrl, null);
+        rpw.setReactingProcess(getCurrentBrowser());//current browser may be null, but it does not metter
+        return rpw.execute();
     }
-    public ProcessResult executeBrowser(Browser b,List<String> otherargs, String resource,ContentReaderListener stdoutl,ContentReaderListener stderrl) throws Exception {
-        return executeProcessUponURL(b.getBin(), otherargs, getUrlUponThisInstance(resource),stdoutl,stderrl);
+
+    public ProcessResult executeBrowser(List<String> otherargs,    String resource, List<ContentReaderListener> stdoutl, List<ContentReaderListener> stderrl) throws Exception {
+        ProcessWrapper rpw = new ProcessWrapper(getBrowserLocation(), otherargs, getUrlUponThisInstance(resource), stdoutl, stderrl, null);
+        rpw.setReactingProcess(getCurrentBrowser());// current browser may be null, but it does not matter
+        return rpw.execute();
+    }
+
+    public ProcessResult executeBrowser(Browser b, List<String> otherargs, String resource) throws Exception {
+        ProcessWrapper rpw = new ProcessWrapper(b.getBin(), otherargs, getUrlUponThisInstance(resource));
+        rpw.setReactingProcess(b);
+        return rpw.execute();
+    }
+
+    public ProcessResult executeBrowser(Browser b, List<String> otherargs, String resource, ContentReaderListener stdoutl, ContentReaderListener stderrl) throws Exception {
+        ProcessWrapper rpw = new ProcessWrapper(b.getBin(), otherargs, getUrlUponThisInstance(resource), stdoutl, stderrl, null);
+        rpw.setReactingProcess(b);
+        return rpw.execute();
+    }
+
+    public ProcessResult executeBrowser(Browser b, List<String> otherargs, String resource, List<ContentReaderListener> stdoutl, List<ContentReaderListener> stderrl) throws Exception {
+        ProcessWrapper rpw = new ProcessWrapper(b.getBin(), otherargs, getUrlUponThisInstance(resource), stdoutl, stderrl, null);
+        rpw.setReactingProcess(b);
+        return rpw.execute();
     }
 
     /**
-     * Ctreate resource on http, on 'localhost' on port on which this cached instance is running
+     * Create resource on http, on 'localhost' on port on which this cached instance is running
      * @param resource
      * @return
      * @throws MalformedURLException
      */
     public URL getUrlUponThisInstance(String resource) throws MalformedURLException {
         getInstance();
-       return getUrlUponInstance(server,resource);
+        return getUrlUponInstance(server, resource);
     }
 
     /**
@@ -630,23 +683,15 @@ public class ServerAccess {
      * @throws Exception
      */
     public static ProcessResult executeProcessUponURL(String toBeExecuted, List<String> otherargs, URL u) throws Exception {
-        return executeProcessUponURL(toBeExecuted, otherargs, u,null,null);
+        return new ProcessWrapper(toBeExecuted, otherargs, u).execute();
     }
 
-    public static ProcessResult executeProcessUponURL(String toBeExecuted, List<String> otherargs, URL u,ContentReaderListener stdoutl,ContentReaderListener stderrl) throws Exception {
-        return executeProcessUponURL(toBeExecuted, otherargs, u, stdoutl, stderrl, null);
+    public static ProcessResult executeProcessUponURL(String toBeExecuted, List<String> otherargs, URL u, ContentReaderListener stdoutl, ContentReaderListener stderrl) throws Exception {
+        return new ProcessWrapper(toBeExecuted, otherargs, u, stdoutl, stderrl, null).execute();
     }
-    public static ProcessResult executeProcessUponURL(String toBeExecuted, List<String> otherargs, URL u,ContentReaderListener stdoutl,ContentReaderListener stderrl,String[] vars) throws Exception {
-        Assert.assertNotNull(u);
-        Assert.assertNotNull(toBeExecuted);
-        Assert.assertTrue(toBeExecuted.trim().length() > 1);
-        if (otherargs == null) {
-            otherargs = new ArrayList<String>(1);
-        }
-        List<String> urledArgs = new ArrayList<String>(otherargs);
-        urledArgs.add(0, toBeExecuted);
-        urledArgs.add(u.toString());
-        return executeProcess(urledArgs, stdoutl, stderrl,vars);
+
+    public static ProcessResult executeProcessUponURL(String toBeExecuted, List<String> otherargs, URL u, ContentReaderListener stdoutl, ContentReaderListener stderrl, String[] vars) throws Exception {
+        return new ProcessWrapper(toBeExecuted, otherargs, u, stdoutl, stderrl, vars).execute();
     }
 
      public static ProcessResult executeProcess(final List<String> args) throws Exception {
@@ -678,10 +723,6 @@ public class ServerAccess {
         return executeProcess(args, dir, null, null);
     }
 
-    private static String createConnectionMessage(ThreadedProcess t) {
-        return "Connecting " + t.getCommandLine();
-    }
-
     /**
      * Proceed message s to logging with request to reprint to System.err
      * @param s
@@ -706,7 +747,7 @@ public class ServerAccess {
         log(s, false, false);
     }
 
-    private static void log(String message, boolean printToOut, boolean printToErr) {
+    static void log(String message, boolean printToOut, boolean printToErr) {
         String idded;
         StackTraceElement ste = getTestMethod();
         String fullId = ste.getClassName() + "." + ste.getMethodName();
@@ -751,26 +792,49 @@ public class ServerAccess {
 
     private static StackTraceElement getTestMethod(StackTraceElement[] stack) {
         //0 is always thread
-        //1 is net.sourceforge.jnlp.ServerAccess
+        //1 is net.sourceforge.jnlp.*
+        //we need to get out of all  of classes from this package or pick last of it
         StackTraceElement result = stack[1];
         String baseClass = stack[1].getClassName();
         int i = 2;
         for (; i < stack.length; i++) {
             result = stack[i];//at least moving up
-            if(stack[i].getClassName().contains("$")){
+            if (stack[i].getClassName().contains("$")) {
                 continue;
             }
-            if (!baseClass.equals(stack[i].getClassName())) {
-                break;
+            //probablky it is necessary to get out of net.sourceforge.jnlp.
+            //package where are right now all test-extensions
+            //for now keeping exactly the three clases helping yo  acces the log
+            try {
+                Class clazz = Class.forName(stack[i].getClassName());
+                String path = null;
+                try {
+                    path = clazz.getProtectionDomain().getCodeSource().getLocation().getPath();
+                } catch (NullPointerException ex) {
+                    //silently ignoring and continuing with null path
+                }
+                if (path != null && path.contains("/tests.build/")) {
+                    if (!path.contains("/test-extensions/")) {
+                        break;
+                    }
+                } else {
+                    //running from ide 
+                    if (!stack[i].getClassName().startsWith("net.sourceforge.jnlp.")) {
+                        break;
+                    }
+                }
+            } catch (ClassNotFoundException ex) {
+                ///should not happen, searching  only for already loaded class
+                ex.printStackTrace();
             }
         }
         //if nothing left in stack then we have been in ServerAccess already
         //so the target method is the highest form it and better to return it
         //rather then die to ArrayOutOfBounds
-        if(i >= stack.length){
+        if (i >= stack.length) {
             return result;
         }
-        //now we are out of net.sourceforge.jnlp.ServerAccess
+        //now we are out of test-extensions
         //method we need (the test)  is highest from following class
         baseClass = stack[i].getClassName();
         for (; i < stack.length; i++) {
@@ -790,48 +854,8 @@ public class ServerAccess {
         return executeProcess(args, dir, stdoutl, stderrl,null);
 
     }
-    public static ProcessResult executeProcess(final List<String> args, File dir, ContentReaderListener stdoutl, ContentReaderListener stderrl,String[] vars) throws Exception {
-        ThreadedProcess t = new ThreadedProcess(args, dir,vars);
-        if (PROCESS_LOG) {
-            String connectionMesaage = createConnectionMessage(t);
-            log(connectionMesaage, true, true);
-        }
-        ProcessAssasin pa = new ProcessAssasin(t, PROCESS_TIMEOUT);
-        pa.start();
-        t.start();
-        while (t.getP() == null && t.deadlyException == null) {
-            Thread.sleep(100);
-        }
-        if (t.deadlyException != null) {
-            pa.setCanRun(false);
-            return new ProcessResult("", "", null, true, Integer.MIN_VALUE, t.deadlyException);
-        }
-        ContentReader crs = new ContentReader(t.getP().getInputStream(),stdoutl);
-        ContentReader cre = new ContentReader(t.getP().getErrorStream(),stderrl);
-
-        OutputStream out = t.getP().getOutputStream();
-        if (out != null) {
-            out.close();
-        }
-
-        new Thread(crs).start();
-        new Thread(cre).start();
-        while (t.isRunning()) {
-            Thread.sleep(100);
-        }
-
-        while (!t.isDestoyed()) {
-            Thread.sleep(100);
-        }
-        pa.setCanRun(false);
-        // ServerAccess.logOutputReprint(t.getP().exitValue()); when process is killed, this throws exception
-
-        ProcessResult pr=new ProcessResult(crs.getContent(), cre.getContent(), t.getP(), pa.wasTerminated(), t.getExitCode(), null);
-        if (PROCESS_LOG) {
-            log(pr.stdout, true, false);
-            log(pr.stderr, false, true);
-        }
-        return pr;
+    public static ProcessResult executeProcess(final List<String> args, File dir, ContentReaderListener stdoutl, ContentReaderListener stderrl, String[] vars) throws Exception {
+        return new ProcessWrapper(args, dir, stdoutl, stderrl, vars).execute();
     }
 
     /**
